package com.gyee.runeconomy.service.WindDirection;

import com.gyee.common.contant.ContantXk;
import com.gyee.common.model.PointData;
import com.gyee.runeconomy.dto.result.PowerPointData;
import com.gyee.runeconomy.init.CacheContext;
import com.gyee.runeconomy.model.auto.ProBasicEquipment;
import com.gyee.runeconomy.model.auto.ProBasicEquipmentPoint;
import com.gyee.runeconomy.service.auto.IProBasicEquipmentPointService;
import com.gyee.runeconomy.util.realtimesource.IEdosUtil;
import com.gyee.runeconomy.util.realtimesource.feign.RemoteServiceBuilder;
import com.gyee.runeconomy.util.realtimesource.feign.TsDoubleData;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class WindDirectionService {

    @Resource
    private IEdosUtil edosUtil;
    @Resource
    private IProBasicEquipmentPointService proBasicEquipmentPointService;
    @Resource
    private RemoteServiceBuilder remoteService;
    /**
     * 风向玫瑰图\风向频率玫瑰图
     *
     * @return
     */
    public Object fxRoses(String wtid, String kssj, String jssj) throws Exception {

        Date[] sjcl = sjcl(kssj, jssj);

        // 获取 startDate 和 endDate
        Date startDate = sjcl[0];
        Date endDate = sjcl[1];

        List<ProBasicEquipment> list = CacheContext.wtls.stream().filter(wt -> wtid.equals(wt.getId())).collect(Collectors.toList());
        if (list == null || list.size() == 0)
            return null;

        ProBasicEquipmentPoint pjfs = null;
        ProBasicEquipmentPoint pjfx = null;
        ProBasicEquipmentPoint sbzt = null;
        ProBasicEquipmentPoint dfwc = null;
        ProBasicEquipmentPoint phjd = null;
        ProBasicEquipmentPoint gl = null;

        pjfs = proBasicEquipmentPointService.getEquipmentPoint(wtid, ContantXk.CJ_SSFS);

        pjfx = proBasicEquipmentPointService.getEquipmentPoint(wtid, ContantXk.CJ_FX);

        sbzt = proBasicEquipmentPointService.getEquipmentPoint(wtid, ContantXk.MXZT);
        List<TsDoubleData> pjfsls = remoteService.adapterfd().getHistorySnap(pjfs.getNemCode(), startDate.getTime(), endDate.getTime(), 900);
        List<TsDoubleData> pjfxls = remoteService.adapterfd().getHistorySnap(pjfx.getNemCode(), startDate.getTime(), endDate.getTime(), 900);
        List<TsDoubleData> sbztls = remoteService.adapterfd().getHistorySnap(sbzt.getNemCode(), startDate.getTime(), endDate.getTime(), 900);
        PointData sbztlSs = edosUtil.getRealData(sbzt);

        List<PowerPointData> lslist = new ArrayList<>();
        for (int i = 0; i < pjfxls.size(); i++) {
            TsDoubleData fxData = pjfxls.get(i);
            TsDoubleData fsData = null;
            TsDoubleData ztData = null;

            for (TsDoubleData fs : pjfsls) {
                if (fxData.getTs() == fs.getTs()) {
                    fsData = fs;
                    break; // 找到匹配的时间戳后退出循环
                }
            }

            for (TsDoubleData zt : sbztls) {
                if (fxData.getTs() == zt.getTs()) {
                    ztData = zt;
                    break; // 找到匹配的时间戳后退出循环
                }
            }

            // 将时间戳转换为 LocalDateTime 对象
            LocalDateTime dateTime = LocalDateTime.ofInstant(Instant.ofEpochMilli(fxData.getTs()), ZoneId.systemDefault());
            // 格式化输出日期
            DateTimeFormatter formatter11 = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String formattedDate = dateTime.format(formatter11);

            // 创建 PowerPointData 对象
            PowerPointData data1 = new PowerPointData();
            data1.setFx(fxData.getDoubleValue());
//            data1.setMxzt(ztData.getPointValueInDouble());
            data1.setTime(formattedDate);

            // 如果没有找到对应的 fsData，默认设置 setFs 为 0
            if (fsData != null) {
                data1.setSpeed(fsData.getDoubleValue());
            } else {
                data1.setSpeed(0);  // 默认值
            }

            // 将 data1 添加到 lslist
            lslist.add(data1);
        }


        List<Object> result = new ArrayList<>();

        for (ProBasicEquipment obj : list) {
            Map<String, Object> map = new HashMap<>();
            map.put("wt", obj.getAname());
            map.put("ys", WindALG.fxNewRoses(lslist));//风速、风向
            map.put("speeedtime", WindALG.speedtimeNewRoses(lslist));//风速
            map.put("roses", WindALG.fxysNewRoses(lslist));//风速、风向
            map.put("count", WindALG.fxNewCountRoses(lslist));//风速、风向
            int[] count = WindALG.windDeviationRatio(lslist);//风向、对风误差
            List<Point> scatter = WindALG.windDeviationScatter(lslist);//风速、风向、对风误差
            map.put("countpcl", count);
            map.put("scatter", scatter);
            map.put("radar", WindALG.fxRadarRoses(lslist));//对风误差
            map.put("frequency", WindALG.windDeviationPoint(lslist)); //风速、风向、明细状态、功率、偏航角度
            result.add(map);
        }

        return result;
    }

    private Date[] sjcl (String kssj,String jssj) throws ParseException {

        // 时间格式，格式是 "yyyy-MM-dd HH:mm"
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        // 将字符串转换为 Date 对象
        Date kssjDate = sdf.parse(kssj);
        Date jssjDate = sdf.parse(jssj);

        // 创建 Calendar 对象
        Calendar kssjCalendar = Calendar.getInstance();
        Calendar jssjCalendar = Calendar.getInstance();

        // 将 Date 设置到 Calendar 对象中
        kssjCalendar.setTime(kssjDate);
        jssjCalendar.setTime(jssjDate);

        // 设置 endDate 为当天的 23:59:59
        jssjCalendar.set(Calendar.HOUR_OF_DAY, 23);
        jssjCalendar.set(Calendar.MINUTE, 59);
        jssjCalendar.set(Calendar.SECOND, 59);
        jssjCalendar.set(Calendar.MILLISECOND, 999);

        // 获取 startDate 和 endDate
        Date startDate = kssjCalendar.getTime();
        Date endDate = jssjCalendar.getTime();
        return new Date[] {startDate, endDate};
    }
}
